@@ -29,7 +29,7 @@
 #include <dbus/dbus-glib-bindings.h>
 
 #define MATE_DESKTOP_USE_UNSTABLE_API
-#include <libmateui/mate-desktop-thumbnail.h>
+#include <libmate-desktop/mate-desktop-thumbnail.h>
 
 #include "e-image-chooser.h"
 #include "mate-about-me-password.h"
@@ -41,8 +41,6 @@
 #define MAX_HEIGHT 100
 #define MAX_WIDTH  100
 
-#define EMAIL_SLOTS 4
-
 typedef struct {
 
 	GtkBuilder 	*dialog;
