@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007,2008 The MATE Foundation
+ * Copyright (C) 2007,2008 The GNOME Foundation
  * Written by Rodney Dawes <dobey@ximian.com>
  *            Denis Washington <denisw@svn.gnome.org>
  *            Thomas Wood <thos@gnome.org>
@@ -27,8 +27,8 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <string.h>
-#include <libmateui/mate-desktop-thumbnail.h>
-#include <libmateui/mate-bg.h>
+#include <libmate-desktop/mate-desktop-thumbnail.h>
+#include <libmate-desktop/mate-bg.h>
 
 enum {
 	TARGET_URI_LIST,
@@ -214,7 +214,11 @@ wp_add_images (AppearanceData *data,
   cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
                                        GDK_WATCH);
   gdk_window_set_cursor (window, cursor);
+#if GTK_CHECK_VERSION (3, 0, 0)
+  g_object_unref (cursor);
+#else
   gdk_cursor_unref (cursor);
+#endif
 
   while (images != NULL)
   {
