@@ -33,6 +33,12 @@
 #include <libappindicator/app-indicator.h>
 #endif /* HAVE_APP_INDICATOR */
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define MATE_DESKTOP_USE_UNSTABLE_API
+#include <libmate-desktop/mate-desktop-utils.h>
+#define gdk_spawn_command_line_on_screen mate_gdk_spawn_command_line_on_screen
+#endif
+
 #include "drwright.h"
 #include "drw-break-window.h"
 #include "drw-monitor.h"
